/* $Id$

This file is part of libmspgl
Copyright © 2007  Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/

#include <IL/il.h>
#include <msp/core/except.h>
#include "ilwrap.h"

using namespace std;

namespace Msp {
namespace GL {

Image::Image()
{
	static bool init_done=false;

	if(!init_done)
	{
		ilInit();
		ilEnable(IL_ORIGIN_SET);
		ilOriginFunc(IL_ORIGIN_LOWER_LEFT);
		init_done=true;
	}

	ilGenImages(1, &id);
}

Image::~Image()
{
	ilDeleteImages(1, &id);
}

void Image::load_file(const string &fn)
{
	ilBindImage(id);
	if(!ilLoadImage(const_cast<char *>(fn.c_str())))
		throw Exception("Error loading image "+fn);
}

void Image::load_lump(const void *data, unsigned size)
{
	ilBindImage(id);
	if(!ilLoadL(IL_TYPE_UNKNOWN, const_cast<void *>(data), size))
		throw Exception("Error loading image from lump");
}

PixelFormat Image::get_format() const
{
	switch(ilGetInteger(IL_IMAGE_FORMAT))
	{
	case IL_COLOR_INDEX: return COLOR_INDEX;
	case IL_LUMINANCE: return LUMINANCE;
	case IL_LUMINANCE_ALPHA: return LUMINANCE_ALPHA;
	case IL_RGB: return RGB;
	case IL_RGBA: return RGBA;
	case IL_BGR: return BGR;
	case IL_BGRA: return BGRA;
	default: throw InvalidParameterValue("Unknown pixel format in image");
	}
}

unsigned Image::get_width() const
{
	return ilGetInteger(IL_IMAGE_WIDTH);
}

unsigned Image::get_height() const
{
	return ilGetInteger(IL_IMAGE_HEIGHT);
}

const void *Image::get_data() const
{
	return ilGetData();
}

} // namespace GL
} // namespace Msp
